/**
  User Application Source File

  Company:
    Microchip Technology Inc.

  File Name:
    app.c

  Summary:
    This is the user application source file which has application specific functions.

  Description:
    This is the user application source file which has functions necessary to execute the user application.

*/

/*
Copyright (c) 2013 - 2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*/

/*Include files*/
#include "app.h"


uint8_t readBuffer[MAX_SONG_SIZE] = {0x00};


#define EEPROM_WRITE_CMD    0x02
#define EEPROM_READ_CMD     0x03
#define EEPROM_PAGE_SIZE    0x40

void ReadSongFromEEPROM(uint8_t* songRead, uint8_t size, uint16_t eepromAddr)
{
    uint8_t eepromCmd[3];
    eepromCmd[0] = EEPROM_READ_CMD;
    eepromCmd[1] = (uint8_t)(eepromAddr >> 8);
    eepromCmd[2] = (uint8_t)(eepromAddr & 0xFF);

    EEPROM_CHIP_SELECT_SetLow();
    SPI1_Exchange8bitBuffer(eepromCmd,3,NULL);            //Send the EEPROM_READ_CMD
    SPI1_Exchange8bitBuffer(NULL, size, songRead);      // Read the actual EEPROM data
    EEPROM_CHIP_SELECT_SetHigh();
}

uint16_t WriteSongToEEPROM( uint8_t* songWrite, uint8_t size, uint16_t eepromAddr)
{


        uint8_t eepromCmd[3];


        eepromCmd[0] = EEPROM_WRITE_CMD;
        eepromCmd[1] = (uint8_t)(eepromAddr >> 8);
        eepromCmd[2] = (uint8_t)(eepromAddr & 0xFF);

        
        while( size )
        {
            // ?? EEPROM ???
            EEPROM_CHIP_SELECT_SetLow();
            SPI1_Exchange8bit(0x06);        // EEPROM ?????? (0x06)
            EEPROM_CHIP_SELECT_SetHigh();
            __delay_ms(1);

            EEPROM_CHIP_SELECT_SetLow();
            SPI1_Exchange8bitBuffer(eepromCmd, 3, NULL);  // ?????????????? (0x0000)

            do
            {

                SPI1_Exchange8bit( *songWrite++ );
                size--;
            }
            while(size);

        }

        EEPROM_CHIP_SELECT_SetHigh();      // disable EEPROM

        return (0x00);

}   // end writeSongToEEPROM()





