/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using MPLAB� Code Configurator

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  MPLAB� Code Configurator - v2.0
        Device            :  PIC18F45K22
        Driver Version    :  2.00
    The generated drivers are tested against the following:
        Compiler          :  XC8 v1.32
        MPLAB             :  MPLAB X 2.10
*/

/*
Copyright (c) 2013 - 2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*/

#include <stdio.h>
#include <string.h>

#include "mcc_generated_files/mcc.h"
#include "app.h"
#include "APP001_LCD.h"

#define I2C_SLAVE_ADDRESS 0x50          // 定義 24LC32A I2C Slave Address 0x05 << 1 後為 0B1010 000b

// #define EEPROM_ADDRESS_HIGH 0x00        // 定義 24LC24A 要存取資料的 MSB Address
#define EEPROM_ADDRESS_LOW 0x00         // 定義 24LC24A 要存取資料的 LSB Address

#define NUM_ADDRESS_BYTES 0x01          // 16-bit 位址架構的 EEPROM。 若為 1 的話為 8-bit 位址的 EEPROM
                                        // 16-bit Address: 24LC32, LC64, LC128, LC256 & LC1024
                                        // 8-bit Address: 24LC02, LC04, LC08, LC16
// 定義寫入的 EEPROM 的陣列資料，(寫入位置 + 資料)
uint8_t eepromWriteBuffer[] = {EEPROM_ADDRESS_LOW,'1', '2', '3', '4','5','6','7','8'};
// 依據"寫入的 EEPROM 的陣列資料" 長度 減去 一個位址資料後, 設定讀取 EEPROM 資料陣列大小
uint8_t eepromReadBuffer[sizeof(eepromWriteBuffer)-NUM_ADDRESS_BYTES] = {0x00};

uint8_t eepromEraseBuffer[sizeof(eepromWriteBuffer)] = {0xFF};


uint8_t i, match;
I2C1_MESSAGE_STATUS i2cStatus;

/*
                         Main application
 */
void main(void)
{

    // Initialize the device
    SYSTEM_Initialize();

    // If using interrupts in PIC18 High/Low Priority Mode you need to enable the Global High and Low Interrupts
    // If using interrupts in PIC Mid-Range Compatibility Mode you need to enable the Global and Peripheral Interrupts
    // Use the following macros to:

    // Enable high priority global interrupts
    //INTERRUPT_GlobalInterruptHighEnable();

    // Enable low priority global interrupts.
    //INTERRUPT_GlobalInterruptLowEnable();

    // Disable high priority global interrupts
    //INTERRUPT_GlobalInterruptHighDisable();

    // Disable low priority global interrupts.
    //INTERRUPT_GlobalInterruptLowDisable();

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();      // 開啟 GIE 中斷 (使用與 PIC16F 相容的中斷模式)

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();  // 開啟 PEIE 中斷

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();
    __delay_ms(100);
    
    /*清除終端機畫面，設定游標位置，顯示操作的方式*/
    printf("\033[2J\033[0;0H");
    printf("\r\nMCC: It's Easy!\r\n\r\nPIC18 Lab4-1: 24LC02B Byte Read/Write Application\r\n");


    printf("\r\nPress switch SW2 to write to the I2C EEPROM\r\n");
    printf("Press switch SW3 to read from the I2C EEPROM\r\n\r\n");

    /*初始設定 LCD 模組，設定游標位置在第一行最左邊位置，顯示 LAB5 Page R/W */
    LCD_L_Delay();              // 10mS
    OpenLCD();
    LCD_Set_Cursor(0,0);
    putrsLCD((const far char*)" Lab4-1 Byte R/W ");
    
    while(1)
    {
        // Add your application code
        if(1 == GetKeyPressed())
        {                                           // 按下 SW2 做 EEPROM 的寫入動作
            pressedKey = 0;
            LCD_Set_Cursor(1,0);
            putrsLCD((const far char*)"EEPROM ByteWrite");

            //將 0xA5 資料寫到 EEPROM 0x30 的位址裡
            eepromWriteBuffer[0] = 0x30;            // 設定寫入位址
            eepromWriteBuffer[1] = 0x61;            // 'a' 設定寫入資料
            I2C1_MasterWrite(eepromWriteBuffer,     // 陣列的指標
                             2,                     // 第一為位址，第二個 Byte 為寫入資料
                             I2C_SLAVE_ADDRESS,     // 0x50 << 1 = 0xA0
                             &i2cStatus);

            //delay to complete I2C Write
            TMR1_EepromDelay();
                        
            printf("EEPROM Write was executed\r\n\r\n");
        }
        else if(2 == GetKeyPressed())
        {                                          // 按下 SW3 做 EEPROM 的讀取動作
            pressedKey = 0;
            LCD_Set_Cursor(1,0);
            putrsLCD((const far char*)"EEPROM Byte Read");

            //設定讀取資料位址為 0x30
            eepromWriteBuffer[0]= 0x30;
            I2C1_MasterWrite(eepromWriteBuffer,
                             1,                     // 長度為 1 , 只寫入位址的設定 (0x30)
                             I2C_SLAVE_ADDRESS,
                             &i2cStatus);

            //I2C Communication delay
            TMR1_EepromDelay();
            
            //讀取一個 Byte EEPROM 的資料
            I2C1_MasterRead(eepromReadBuffer,1,I2C_SLAVE_ADDRESS, &i2cStatus);

            printf("EEPROM Read was executed\r\n");
            printf("Data Match SUCCESS\r\n");
            printf("EEPROM DATA is: ");
            printf("%c",eepromReadBuffer[0]);
            printf("\r\n");
        
        }        
    }
}

/**
 End of File
*/
