/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using MPLAB� Code Configurator

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  MPLAB� Code Configurator - v2.0
        Device            :  PIC18F45K22
        Driver Version    :  2.00
    The generated drivers are tested against the following:
        Compiler          :  XC8 v1.32
        MPLAB             :  MPLAB X 2.10
*/

/*
Copyright (c) 2013 - 2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "mcc_generated_files/mcc.h"
#include "app.h"
#include "APP001_LCD.h"

unsigned Read_TC74_Temperature(void);
void Delay_200mS(void);

union
{
	int	Word;
	char Byte[2];

	struct{
			unsigned:6 ;
			unsigned BitD6:1;
		   };
} TC_74;

#define TC74_Addr	0b01001111			// Define the TC74-A7 address with write command
#define TC74_RWCR	0x01				// Define the Read/Write Configuration Command
#define TC74_RTR	0x00				// Define the read temperature command

#define NUM_ADDRESS_BYTES 0x01          

// 定義寫入的 EEPROM 的陣列資料，(寫入位置 + 資料)
uint8_t TC74WriteBuffer[3] = {0x00};

uint8_t TC74ReadBuffer[2] = {0x00};

char ASCII_String[10];

unsigned char T1_Buffer;
unsigned char i, match;


/*
                         Main application
 */
void main(void)
{

    // Initialize the device
    SYSTEM_Initialize();

    // If using interrupts in PIC18 High/Low Priority Mode you need to enable the Global High and Low Interrupts
    // If using interrupts in PIC Mid-Range Compatibility Mode you need to enable the Global and Peripheral Interrupts
    // Use the following macros to:

    // Enable high priority global interrupts
    //INTERRUPT_GlobalInterruptHighEnable();

    // Enable low priority global interrupts.
    //INTERRUPT_GlobalInterruptLowEnable();

    // Disable high priority global interrupts
    //INTERRUPT_GlobalInterruptHighDisable();

    // Disable low priority global interrupts.
    //INTERRUPT_GlobalInterruptLowDisable();

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();      // 開啟 GIE 中斷 (使用與 PIC16F 相容的中斷模式)

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();  // 開啟 PEIE 中斷

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();
    __delay_ms(100);
    
    /*清除終端機畫面，設定游標位置，顯示操作的方式*/
    printf("\033[2J\033[0;0H");
    printf("\r\nMCC: It's Easy!\r\n\r\nPIC18 Lab4-2: Read the Temperature Sensor of TC74 \r\n");


     /*初始設定 LCD 模組，設定游標位置在第一行最左邊位置，顯示 Lab5-2 Read TC74 */
    LCD_L_Delay();              // 10mS
    OpenLCD();
    LCD_Set_Cursor(0,0);
    putrsLCD((const far char*)"Lab4-2 Read TC74");

    while (1)
    {
        Read_TC74_Temperature( );               // 讀取 TC74 的溫度
        T1_Buffer= Read_TC74_Temperature( );  	// Put the Temperature 1 on LCD
        itoa(ASCII_String,T1_Buffer,10);
        
        LCD_Set_Cursor(1,0);
        putrsLCD("Temp. = ");
        putsLCD(ASCII_String);
        putcLCD(0xDF);                          // Add degree symbol
        putrsLCD("C  ");

        printf("\033[2K");                      // delete the current line
        printf("\033[6;0H");                    // set cursor to the beginning of the current line
        printf("\r\nRead TC74 Temperature = %sC\r\n",ASCII_String);
        
        __delay_ms(200);                         // TC74 轉換速度較慢，延遲等 200mS 後再讀取
        LED_D8_Toggle();
    }
}



unsigned Read_TC74_Temperature(void)
//*************************************************************** 
//***      Read a Temperature Hex Value for the TC74         ****
//*************************************************************** 
//* The function will read a configuration register first.      *
//* Check the Data Ready bit (b6) in the Config Reg.            *
//* B6=0, Not Ready, Return (-2)                                *
//* B6=1, Data is Ready, read the temperature value             *
//*                                                             *
//* Return Value = >0 ; Normal Temperature Read                 *
//* Return Value = -1 ; Fail to read temperature                *
//* Return Value = -2 ; TC74-A7 device is not ready             * 
//*                                                             *
//*  TC74 Accuracy +-2C , From 25 C~ 85C                        *
//*  Out Hex Value :  0C --> 0x00	  1C --> 0x01               *
//*		 25C --> 0x19		127C --> 0x7F                       *
//*		-25C --> 0xE7		-55C --> 0xC9                       *
//*************************************************************** 
{
    I2C1_MESSAGE_STATUS i2cStatus;

        TC_74.Word = 0 ;
        // 用 Random Read 方式讀取 TC74 的 狀態值  
        TC74WriteBuffer[0]= TC74_RWCR;            
        I2C1_MasterWrite(TC74WriteBuffer,               // 寫入 Config. 暫存器位址 (0x01)
                             1,                         // 長度為 1 , 只寫入位址的設定
                             TC74_Addr,
                             &i2cStatus);
        I2C1_MasterRead(TC74ReadBuffer,1,TC74_Addr, &i2cStatus);
        TC_74.Word = TC74ReadBuffer[0];

	if ( TC_74.BitD6 )                                  // b6 =1 , Read temperature
	{
        // 用 Random Read 方式讀取 TC74 的 溫度值
        TC74WriteBuffer[0]= TC74_RTR;            
        I2C1_MasterWrite(TC74WriteBuffer,               // 寫入 Config. 暫存器位址 (0x00)
                         1,                             // 長度為 1 , 只寫入位址的設定 
                         TC74_Addr,
                         &i2cStatus);
        I2C1_MasterRead(TC74ReadBuffer,1,TC74_Addr, &i2cStatus);
        TC_74.Word = TC74ReadBuffer[0];
                
		if ( TC_74.Word >= 0 ) return TC_74.Word +1;    // + 1 度 C
		else return -1;                                 // 負溫度, return (-1)
	}
	else return -2;                                     // b6=0, TC74 處於忙碌狀態中 return (-2)
}

/**
 End of File
*/
