/**
  Generated Main Source File

  Company:
    Microchip Technology Inc.

  File Name:
    main.c

  Summary:
    This is the main file generated using MPLAB� Code Configurator

  Description:
    This header file provides implementations for driver APIs for all modules selected in the GUI.
    Generation Information :
        Product Revision  :  MPLAB� Code Configurator - v2.0
        Device            :  PIC18F45K22
        Driver Version    :  2.00
    The generated drivers are tested against the following:
        Compiler          :  XC8 v1.32
        MPLAB             :  MPLAB X 2.10
*/

/*
Copyright (c) 2013 - 2014 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*/

#include <stdio.h>
#include <string.h>

#include "mcc_generated_files/mcc.h"
#include "app.h"
#include "APP001_LCD.h"

#define I2C_SLAVE_ADDRESS 0x50          // 定義 24LC32A I2C Slave Address 0x05 << 1 後為 0B1010 000b

#define EEPROM_ADDRESS_HIGH 0x00        // 定義 24LC24A 要存取資料的 MSB Address
#define EEPROM_ADDRESS_LOW 0x00         // 定義 24LC24A 要存取資料的 LSB Address

#define NUM_ADDRESS_BYTES 0x02          // 16-bit 位址架構的 EEPROM。 若為 1 的話為 8-bit 位址的 EEPROM
                                        // 16-bit Address: 24LC32, LC64, LC128, LC256 & LC1024
                                        // 8-bit Address: 24LC02, LC04, LC08, LC16
// 定義寫入的 EEPROM 的陣列資料，前兩個 Bytes 為 EEPROM 的寫入位址，第三個以後才是寫入資料。
uint8_t eepromWriteBuffer[] = {EEPROM_ADDRESS_HIGH, EEPROM_ADDRESS_LOW, 'G', 'O', 'O', 'D', ' ', 'D', 'A', 'Y', ' ', '!'};
// 依據"寫入的 EEPROM 的陣列資料" 長度 減去 兩個位址資料後, 設定讀取 EEPROM 資料陣列大小
uint8_t eepromReadBuffer[sizeof(eepromWriteBuffer)-NUM_ADDRESS_BYTES] = {0x00};

uint8_t eepromEraseBuffer[sizeof(eepromWriteBuffer)] = {0xFF};


uint8_t i, match;
I2C1_MESSAGE_STATUS i2cStatus;

/*
                         Main application
 */
void main(void)
{

    // Initialize the device
    SYSTEM_Initialize();

    // If using interrupts in PIC18 High/Low Priority Mode you need to enable the Global High and Low Interrupts
    // If using interrupts in PIC Mid-Range Compatibility Mode you need to enable the Global and Peripheral Interrupts
    // Use the following macros to:

    // Enable high priority global interrupts
    //INTERRUPT_GlobalInterruptHighEnable();

    // Enable low priority global interrupts.
    //INTERRUPT_GlobalInterruptLowEnable();

    // Disable high priority global interrupts
    //INTERRUPT_GlobalInterruptHighDisable();

    // Disable low priority global interrupts.
    //INTERRUPT_GlobalInterruptLowDisable();

    // Enable the Global Interrupts
    INTERRUPT_GlobalInterruptEnable();      // 開啟 GIE 中斷 (使用與 PIC16F 相容的中斷模式)

    // Enable the Peripheral Interrupts
    INTERRUPT_PeripheralInterruptEnable();  // 開啟 PEIE 中斷

    // Disable the Global Interrupts
    //INTERRUPT_GlobalInterruptDisable();

    // Disable the Peripheral Interrupts
    //INTERRUPT_PeripheralInterruptDisable();
    __delay_ms(100);
    
    /*清除終端機畫面，設定游標位置，顯示操作的方式*/
    printf("\033[2J\033[0;0H");
    printf("\r\nMCC: It's Easy!\r\n\r\nPIC18 Lab 4: Access EEPROM 24LC32A\r\n");

    printf("\r\nPress switch SW2 to write to the I2C EEPROM\r\n");
    printf("Press switch SW3 to read from the I2C EEPROM\r\n\r\n");

    /*初始設定 LCD 模組，設定游標位置在第一行最左邊位置，顯示 LAB5 Page R/W */
    LCD_L_Delay();              // 10mS
    OpenLCD();
    LCD_Set_Cursor(0,0);
    putrsLCD((const far char*)" Lab4 Page R/W ");
    
    while(1)
    {
        // Add your application code
        if(1 == GetKeyPressed())
        {                                           // 按下 SW2 做 EEPROM 的寫入動作
            pressedKey = 0;
            LCD_Set_Cursor(1,0);
            putrsLCD((const far char*)"Write to EEPROM!");

            //將資料寫到 24LC32A EEPROM
            //TODO: 將陣列 eepromWriteBuffer的資料，寫入資料的長度，24LC32A Slave Address 及狀態回傳變數 (共四項)
            //      使用 I2C1_MasterWrite( ) 函數來實現資料讀寫入
//            I2C1_MasterWrite(eepromWriteBuffer,sizeof(eepromWriteBuffer),I2C_SLAVE_ADDRESS,&i2cStatus);

            //delay to complete I2C Write
            TMR1_EepromDelay();
                        
            printf("EEPROM Write was executed\r\n\r\n");
        }
        else if(2 == GetKeyPressed())
        {                                          // 按下 SW3 做 EEPROM 的讀取動作
            pressedKey = 0;
            LCD_Set_Cursor(1,0);
            putrsLCD((const far char*)"Read from EEPROM");

            //將欲讀取資料的位址寫到 24LC32A EEPROM
            //TODO: 將陣列 eepromWriteBuffer的資料，幾個位址長度 (2)，24LC32A Slave Address 及狀態回傳變數 (共四項)
            //      使用 I2C1_MasterWrite( ) 函數來實現位址的設定
//            I2C1_MasterWrite(eepromWriteBuffer,NUM_ADDRESS_BYTES,I2C_SLAVE_ADDRESS,&i2cStatus);

            //I2C Communication delay
            TMR1_EepromDelay();
            
            //依據所輸入的讀取長度來讀取 EEPROM 的資料
            //TODO: 將陣列 eepromReadBuffer的資料，讀取資料個數，24LC32A Slave Address 及狀態回傳變數 (共四項)
            //      使用 I2C1_MasterRead( ) 函數來實現資料的讀取
//            I2C1_MasterRead(eepromReadBuffer,sizeof(eepromReadBuffer),I2C_SLAVE_ADDRESS, &i2cStatus);
            printf("EEPROM Read was executed\r\n");

            match = true;
            for(i = 0; i < (sizeof(eepromReadBuffer)); i++)      // 回讀資料與寫入資料做比較
            {
                if (eepromWriteBuffer[i+NUM_ADDRESS_BYTES] != eepromReadBuffer[i])
                {
                    match = false;          // 比較錯誤，設定錯誤旗幟
                    break;
                }
            }

            if(match)
            {                               // 比對正確，顯示 Data Match SUCCESS ...
                printf("Data Match SUCCESS\r\n");              
                printf("EEPROM DATA is: ");

                for(i=0; i<(sizeof(eepromReadBuffer)); i++)
                {
                     printf("%c",eepromReadBuffer[i]);
                }
                printf("\r\n");

            }
            else                            // 比對錯誤，顯示 Data Match FAILURE ...
            {
                printf("Data Match FAILURE - press SW2 to write to EEPROM and try again\r\n");
            }
            printf("\r\n");

        }        
    }
}

/**
 End of File
*/
