/*******************************************************************************
  Music Demonstration File 

  Company:
    Microchip Technology Inc.

  File Name:
    music.c

  Summary:
    This file plays music on the PICtail Plus Speech Playback board

*******************************************************************************/

/*******************************************************************************
Copyright (c) 2013 released Microchip Technology Inc.  All rights reserved.

Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*******************************************************************************/

#include "mcc_generated_files/mcc.h"
#include <stdint.h>
#include "music.h"



/**
  This structure holds the definition of period and duty cycle for musical notes
*/
typedef struct  note_t
{
    uint8_t     PR;
    uint16_t    CCPR;
    uint8_t     note;
} note_t;



/**
  This array hold the musical notes and rests to play the song.
  The notes are expressed as hexadecimal numbers.
  0xC4 = middle C
  0xD4 = middle D
  and so on.

  The note 'G' is mapped to '0', so middle G = 0x04

  Rests are represented by 0xFF

  The end of the song is identified by a 0x00
*/

uint8_t rowRow[] =
{
    0xC4,
    0xC4,
    0xC4,
    0xFF,
    0xC4,
    0xC4,
    0xC4,
    0xFF,
    0xC4,
    0xC4,
    0xD4,
    0xFF,
    0xE4,
    0xE4,
    0xE4,
    0xFF,
    0xE4,
    0xE4,
    0xD4,
    0xFF,
    0xE4,
    0xE4,
    0xF4,
    0xFF,
    0x04,
    0x04,
    0x04,
    0x04,
    0x04,
    0x04,
    0xFF,
    0xFF,
    0xFF,
    0xC5,
    0xC5,
    0xC5,
    0xFF,
    0x04,
    0x04,
    0x04,
    0xFF,
    0xE4,
    0xE4,
    0xE4,
    0xFF,
    0xC4,
    0xC4,
    0xC4,
    0xFF,
    0xFF,
    0xFF,
    0x04,
    0x04,
    0xC4,
    0xFF,
    0xE4,
    0xE4,
    0xD4,
    0xFF,
    0xC4,
    0xC4,
    0xC4,
    0xC4,
    0xC4,
    0xC4,
    0xFF,
    0xFF,
    0xFF,
    0xFF,
    0xFF,
    0xFF,
    0xFF,
    0xFF,
    0x00
}; // end rowRow[]

uint8_t rowRowSize =  16; //sizeof(rowRow);

/**
  The noteTable contains the PWM period (PR) and duty cycle (CCPR)
  information for each note.  As the music is played, the the
  desired note is looked up in this table, and the PWM period and
  duty cycle listed are used to produce the note.
*/
note_t     NOTE_TABLE[] =
{
    /*    PR      CCPR    Note         Period       Frequency   */
        { 252,    506,    0xB1 },   // 16.197E-3    61.74
        { 237,    476,    0xC2 },   // 15.288E-3    65.41
        { 211,    424,    0xD2 },   // 13.620E-3    73.42
        { 188,    378,    0xE2 },   // 12.134E-3    82.41
        { 177,    356,    0xF2 },   // 11.453E-3    87.31
        { 158,    318,    0x02 },   // 10.204E-3    98
        { 141,    284,    0xA2 },   // 9.091E-3     110
        { 125,    252,    0xB2 },   // 8.099E-3     123.47
        { 118,    238,    0xC3 },   // 7.645E-3     130.81
        { 105,    212,    0xD3 },   // 6.811E-3     146.83
        { 93,     188,    0xE3 },   // 6.068E-3     164.81
        { 88,     178,    0xF3 },   // 5.727E-3     174.61
        { 78,     158,    0x03 },   // 5.102E-3     196
        { 70,     142,    0xA3 },   // 4.545E-3     220
        { 62,     126,    0xB3 },   // 4.050E-3     246.94
        { 58,     118,    0xC4 },   // 3.822E-3     261.63
        { 52,     106,    0xD4 },   // 3.405E-3     293.66
        { 46,     94,     0xE4 },   // 3.034E-3     329.63
        { 43,     88,     0xF4 },   // 2.863E-3     349.23
        { 38,     78,     0x04 },   // 2.551E-3     392
        { 34,     70,     0xA4 },   // 2.273E-3     440
        { 30,     62,     0xB4 },   // 2.025E-3     493.88
        { 28,     58,     0xC5 },   // 1.911E-3     523.25
        { 25,     52,     0xD5 },   // 1.703E-3     587.33
        { 22,     46,     0xE5 },   // 1.517E-3     659.26
        { 21,     44,     0xF5 },   // 1.432E-3     698.46
        { 18,     38,     0x05 },   // 1.276E-3     783.99
        { 16,     34,     0xA5 },   // 1.136E-3     880
        { 14,     30,     0xB5 },   // 1.012E-3     987.77
        { 0,      0,      0xFF },   // rest
        { 0,      0,      0    },   // null terminator
}; // end NOTE_TABLE[]




uint8_t MusicNoteLookup(uint8_t searchNote)
{
    uint8_t tableNote   = 0;


    // step through the note table looking for the note
    while(NOTE_TABLE[tableNote].note)
    {
        if(NOTE_TABLE[tableNote].note == searchNote)
        {
            break;
        }
        tableNote++;
    }

    return tableNote;

} // end MusicNoteLookup()


void MusicPlay(uint8_t* find)
{
    uint8_t             play;

    play = MusicNoteLookup( *find );

    TMR2_LoadPeriodRegister(NOTE_TABLE[play].PR);
    EPWM1_LoadDutyValue(NOTE_TABLE[play].CCPR);

}

